@@ -35,6 +35,7 @@ use OAR::Conf qw(init_conf dump_conf get_conf_list get_conf is_conf set_value);
 use OAR::IO;
 use OAR::Stat;
 use OAR::Nodes;
+use OAR::Walltime;
 use OAR::Tools;
 use OAR::Version;
 use POSIX;
@@ -227,7 +228,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /: Root links
   #
-  $URI = qr{^\/*\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
@@ -280,7 +281,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /version : Version informations
   #
-  $URI = qr{^/version\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('version');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
@@ -299,7 +300,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /whoami : Authenticated User informations
   #
-  $URI = qr{^/whoami\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('whoami');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
@@ -315,7 +316,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /timezone: Timezone information
   #
-  $URI = qr{^/timezone\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('timezone');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
@@ -337,7 +338,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /jobs[/details|table]?state=<state>,from=<from>,to=<to>,ids=<id1:id2:...> : List of jobs
   #
-  $URI = qr{^/jobs(/details|/table)*\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('jobs(/details|/table)?');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext=OAR::API::set_ext($q,$2);
@@ -353,6 +354,8 @@ SWITCH: for ($q) {
     if ( $authenticated_user =~ /(\w+)/ ) {
       $authenticated_user = $1;
       $ENV{OARDO_USER} = $authenticated_user;
+    }else{
+      $ENV{OARDO_USER} = '';
     }
 
     OAR::Stat::open_db_connection or OAR::API::ERROR(500, 
@@ -432,7 +435,7 @@ SWITCH: for ($q) {
                   # SQL queries. Maybe to optimize...
                   my $detailed_jobs;
                   foreach my $j (@$jobs) {
-                    my $job_resources = OAR::Stat::get_job_resources($j);
+                    my $job_resources = OAR::API::get_job_resources($j);
                     $j = OAR::Stat::get_job_data($j,1);
                     my $resources = OAR::API::struct_job_resources($job_resources,$STRUCTURE);
                     my $nodes= OAR::API::struct_job_nodes($job_resources,$STRUCTURE);
@@ -465,13 +468,13 @@ SWITCH: for ($q) {
   #
   #{{{ GET /jobs/<id>[/details] : Infos of a job. Adding /details results in a "oarstat -f" equivalent
   #
-  $URI = qr{^/jobs/(\d+)(/details)*(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('jobs/(\d+)(/details)?');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $jobid = $1;
     my $details = $2;
     my $ext=OAR::API::set_ext($q,$3);
-    (my $header, my $type)=OAR::API::set_output_format($ext,"GET, POST, DELETE");
+    (my $header, my $type)=OAR::API::set_output_format($ext,"GET, POST, PUT, DELETE");
     
     # Must be authenticated
     if ( not $authenticated_user =~ /(\w+)/ ) {
@@ -493,22 +496,25 @@ SWITCH: for ($q) {
         "Job not found" );
       last;
     }
-    my $j=OAR::Stat::get_job_data(@$job[0],1);
+    my $data=OAR::Stat::get_job_data(@$job[0],1);
     if (defined($details) and $details eq "/details") {
-        my $job_resources = OAR::Stat::get_job_resources(@$job[0]);
+        my $job_resources = OAR::API::get_job_resources(@$job[0]);
         my $resources = OAR::API::struct_job_resources($job_resources,$STRUCTURE);
         my $nodes= OAR::API::struct_job_nodes($job_resources,$STRUCTURE);
         OAR::API::add_resources_uris($resources,$ext,'');
-        $j->{'resources'}=$resources;
+        $data->{'resources'}=$resources;
         OAR::API::add_nodes_uris($nodes,$ext,'');
-        $j->{'nodes'}=$nodes;
+        $data->{'nodes'}=$nodes;
     }
-
-    my $result = OAR::API::struct_job($j,$STRUCTURE);
+    my ($walltime_change,) = OAR::Stat::get_job_walltime_change($jobid);
+    if (defined($walltime_change)) {
+        $data->{'walltime-change'} = $walltime_change;
+    }
+    my $result = OAR::API::struct_job($data,$STRUCTURE);
     OAR::API::add_job_uris($result,$ext);
     OAR::Stat::close_db_connection; 
     print $header;
-    if ($ext eq "html") { OAR::API::job_html_header($j); };
+    if ($ext eq "html") { OAR::API::job_html_header($data); };
     print OAR::API::export($result,$ext);
     last;
   };
@@ -516,7 +522,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /jobs/<id>/[resources|nodes] : Resources or nodes assigned or scheduled to a job
   #
-  $URI = qr{^/jobs/(\d+)/(resources|nodes)(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('jobs/(\d+)/(resources|nodes)');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $jobid = $1;
@@ -528,7 +534,7 @@ SWITCH: for ($q) {
                                                 "Cannot connect to the database"
                                                  );
     my $job = OAR::Stat::get_specific_jobs([$jobid]);
-    my $resources=OAR::Stat::get_job_resources(@$job[0]);
+    my $resources=OAR::API::get_job_resources(@$job[0]);
     if ($item eq "resources") {
       $resources = OAR::API::struct_job_resources($resources,$STRUCTURE);
       OAR::API::add_resources_uris($resources,$ext,''); 
@@ -547,7 +553,7 @@ SWITCH: for ($q) {
   #
   #{{{ POST /jobs/[array/]<id>/checkpoints|deletions|holds|rholds|resumptions|resubmissions/new : Actions on a job (checkpoint, hold, resume,...)
   #
-  $URI = qr{^/jobs/(array/|)(\d+)/(checkpoints|deletions|holds|rholds|resumptions|resubmissions)+/new(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('jobs/(array/)?(\d+)/(checkpoints|deletions|holds|rholds|resumptions|resubmissions)/new');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $array = $1;
@@ -654,12 +660,11 @@ SWITCH: for ($q) {
   # Should not be used unless for delete from an http browser
   # (better to use the URI above)
   #
-  $URI = qr{^/jobs/(\d+)(\.yaml|\.json|\.html)*$};
-  OAR::API::POST( $_, $URI ) && do {
-    $_->path_info =~ m/$URI/;
+  $URI = OAR::API::uri_regex_html_json_yaml('jobs/(\d+)');
+  (OAR::API::PUT( $_, $URI ) || OAR::API::POST( $_, $URI )) && do { $_->path_info =~ m/$URI/;
     my $jobid = $1;
     my $ext=OAR::API::set_ext($q,$2);
-    (my $header, my $type)=OAR::API::set_output_format($ext,"GET, POST");
+    (my $header, my $type)=OAR::API::set_output_format($ext,"GET, POST, PUT, DELETE");
  
      # Must be authenticated
     if ( not $authenticated_user =~ /(\w+)/ ) {
@@ -675,9 +680,9 @@ SWITCH: for ($q) {
     my $job;
     if ($q->param('POSTDATA')) {
       $job = OAR::API::check_job_update( $q->param('POSTDATA'), $q->content_type );
-    }
-    # From html form
-    else {
+    } elsif ($q->param('PUTDATA')) {
+      $job = OAR::API::check_job_update( $q->param('PUTDATA'), $q->content_type );
+    } else {# From html form
       $job = OAR::API::check_job_update( $q->Vars, $q->content_type );
     }
     
@@ -702,17 +707,48 @@ SWITCH: for ($q) {
       $cmd    = "$OARDODO_CMD $OARRESUME_CMD $jobid";
       $status = "Resume request registered";
     }
+    # Signal
+    elsif ( $job->{method} eq "signal" ) {
+      $cmd    = "$OARDODO_CMD $OARDEL_CMD -s $job->{signal} $jobid";
+      $status = "Signal sending request registered"; 
+    }
+    # Walltime change
+    elsif ( $job->{method} eq "walltime-change" ) {
+      # nothing here, see below
+    }
     else {
       OAR::API::ERROR(400,"Bad query","Could not understand ". $job->{method} ." method"); 
       last;
     }
 
-    my $cmdRes = OAR::API::send_cmd($cmd,"Oar");
-    print $q->header( -status => 202, -type => "$type" );
+    my $message;
+    my $http_status;
+    if ( $job->{method} eq "walltime-change" ) {
+      # Check and get the submitted data
+      # From encoded data
+      my $dbh = OAR::IO::connect() or OAR::API::ERROR(500,
+                                                "Cannot connect to the database",
+                                                "Cannot connect to the database"
+                                                 );
+      my $error;
+      ($error, $http_status, $status, $message) =
+        OAR::Walltime::request($dbh, $jobid, $authenticated_user, $job->{walltime}, $job->{force}, $job->{'delay_next_jobs'});
+      OAR::IO::disconnect($dbh);
+
+      if ($error > 0) {
+        OAR::API::ERROR($http_status, ucfirst($status), ucfirst($message));
+      }
+    }
+    else {
+      $http_status = 202;
+      $message = OAR::API::send_cmd($cmd,"Oar");
+    }
+ 
+    print $q->header( -status => $http_status, -type => "$type" );
     print $HTML_HEADER if ($ext eq "html");
     print OAR::API::export( { 'id' => "$jobid",
-                    'status' => "$status",
-                    'cmd_output' => "$cmdRes",
+                    'status' => ucfirst($status),
+                    'cmd_output' => ucfirst($message),
                     'api_timestamp' => time(),
                     'links' => [ { 'rel' => 'self' , 'href' => 
                           OAR::API::htmlize_uri(OAR::API::make_uri("jobs/$jobid",$ext,0),$ext) } ]
@@ -723,7 +759,7 @@ SWITCH: for ($q) {
   #
   #{{{ POST /jobs/<id>/signals/<signal> : Signal sending
   #
-  $URI = qr{^/jobs/(\d+)/signals/(\d+)(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('jobs/(\d+)/signals/(\d+)');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $jobid = $1;
@@ -757,7 +793,7 @@ SWITCH: for ($q) {
   #
   #{{{ POST /jobs : A new job (oarsub wrapper)
   #
-  $URI = qr{^/jobs\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('jobs');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext=OAR::API::set_ext($q,$1);
@@ -927,12 +963,12 @@ SWITCH: for ($q) {
   #
   #{{{ DELETE /jobs/<id> : Delete a job (oardel wrapper)
   #
-  $URI = qr{^/jobs/(\d+)(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('jobs/(\d+)');
   OAR::API::DELETE( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $jobid = $1;
     my $ext=OAR::API::set_ext($q,$2);
-    (my $header, my $type)=OAR::API::set_output_format($ext);
+    (my $header, my $type)=OAR::API::set_output_format($ext,"GET, POST, DELETE");
 
     # Must be authenticated
     if ( not $authenticated_user =~ /(\w+)/ ) {
@@ -958,7 +994,7 @@ SWITCH: for ($q) {
   #
   #{{{       /jobs/stagein and stageout (desktop computing)
   #
-  $URI = qr{^/jobs/(\d+)/stagein(.tar\.gz|.tgz)$};
+  $URI = OAR::API::uri_regex_tgz('jobs/(\d+)/stagein');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$2);
@@ -968,7 +1004,7 @@ SWITCH: for ($q) {
     last;
   };
 
-  $URI = qr{^/jobs/(\d+)/stagein(.tar\.gz|.tgz)$};
+  $URI = OAR::API::uri_regex_tgz('jobs/(\d+)/stageinhead');
   OAR::API::HEAD( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$2);
@@ -977,7 +1013,7 @@ SWITCH: for ($q) {
     last;
   };
 
-  $URI = qr{^/jobs/(\d+)/stageout(.tar\.gz|.tgz)$};
+  $URI = OAR::API::uri_regex_tgz('jobs/(\d+)/stageout');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$2);
@@ -998,19 +1034,25 @@ SWITCH: for ($q) {
   #
   #{{{ POST /jobs/<id>/state : changes the state of a job
   #
-  $URI = qr{^/jobs/(\d+)/state(.*)$};
+  $URI = OAR::API::uri_regex_html_json_yaml('jobs/(\d+)/state');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$2);
     (my $header, my $type)=OAR::API::set_output_format($ext,"GET, POST");
     print $q->header( -status => 200, -type => $type );
-    my $json = decode_json $q->param('POSTDATA');
-    my $state = $json->{'state'};
-    if ($state eq 'running'){
+    my $state;
+    if ($q->param('POSTDATA')) {
+      $state = OAR::API::check_state( $q->param('POSTDATA'), $q->content_type );
+    }
+    # From html form
+    else {
+      $state = OAR::API::check_state( $q->Vars, $q->content_type );
+    }
+    if ($state->{'state'} eq 'running'){
         OAR::API::runJob($1);
-    } elsif($state eq 'terminated'){
+    } elsif($state->{'state'} eq 'terminated'){
         OAR::API::terminateJob($1);
-    } elsif($state eq 'error'){
+    } elsif($state->{'state'} eq 'error'){
         OAR::API::errorJob($1);
     } else {
         die "unknown state"
@@ -1025,7 +1067,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /resources/(details|<id>) : List of resources or details of a resource
   #
-  $URI = qr{^/resources(/full|/details|/[0-9]+)*\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('resources(/full|/details|/\d+)?');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext=OAR::API::set_ext($q,$2);
@@ -1118,7 +1160,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /resources/nodes/<node> : List the resources of a node
   #
-  $URI = qr{^/resources/nodes/([\w\.-]+?)(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('resources/nodes/([\w.-]+?)');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext=OAR::API::set_ext($q,$2);
@@ -1142,7 +1184,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /resources/(<id>)/jobs : Jobs running on a resource
   #
-  $URI = qr{^/resources(/[0-9]+)+/jobs(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('resources(/\d+)/jobs');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext=OAR::API::set_ext($q,$2);
@@ -1170,7 +1212,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /resources/nodes/<node>/jobs : Jobs running on a node
   #
-  $URI = qr{^/resources/nodes/([-\.\w]+)/jobs(.*)$};
+  $URI = OAR::API::uri_regex_html_json_yaml('resources/nodes/([\w.-]+)/jobs');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$2);
@@ -1221,7 +1263,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /resources/<property>/jobs : List jobs running on resources, by property
   #
-  $URI = qr{^/resources/([A-za-z0-9]+)*/jobs\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('resources/([\w-]+)/jobs');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext=OAR::API::set_ext($q,$2);
@@ -1269,7 +1311,7 @@ SWITCH: for ($q) {
   #
   #{{{ POST /resources : Create new resources
   # 
-  $URI = qr{^/resources(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('resources');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext=OAR::API::set_ext($q,$1);
@@ -1328,7 +1370,7 @@ SWITCH: for ($q) {
   #
   #{{{ POST /resources/<id>/state : Change the state of a resource
   # 
-  $URI = qr{^/resources/(\d+)/state(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('resources/(\d+)/state');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $id=$1;
@@ -1381,7 +1423,7 @@ SWITCH: for ($q) {
   #
   #{{{ DELETE /resources/(<id>|<node>/<cpuset) : Delete a resource (by id or node+cpuset)
   #
-  $URI = qr{^/resources/([\w\.-]+?)(/\d)*(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('resources/(?:(\d+)|([\w.-]+)/(\d+))');
   OAR::API::DELETE( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $id;
@@ -1470,7 +1512,7 @@ SWITCH: for ($q) {
   # 
   #{{{ POST /resources/generate : Generate resources (needs an external command like oar_resources_add)
   #
-  $URI = qr{^/resources/generate(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('resources/generate');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
@@ -1534,7 +1576,7 @@ SWITCH: for ($q) {
     $cmd = "$API_RESOURCES_LIST_GENERATION_CMD $auto_offset".$description->{resources}.$cmd_properties;
     # execute the command
     my $cmdRes = OAR::API::send_cmd($cmd,"Oar");
-    my $data = OAR::API::import_data($cmdRes,"yaml");
+    my $data = OAR::API::import_data_with_format($cmdRes,"yaml");
     OAR::API::struct_resource_list_fix_ints($data);
     $data = OAR::API::add_pagination($data,@$data,$q->path_info,undef,$ext,0,0,$STRUCTURE);
     print $header;
@@ -1553,7 +1595,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /accounting[<user>]?from=<from>,to=<to> : Show accounting informations between 2 unix timestamps
   #
-  $URI = qr{^/accounting(/[^\..]+)*\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('accounting(/.+?)');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
 
@@ -1623,7 +1665,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /admission_rules : List of all admissions rules
   #
-  $URI = qr{^/admission_rules\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('admission_rules');
   OAR::API::GET( $_, $URI ) && do {
   	$_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
@@ -1667,7 +1709,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /admission_rules/<id> : Details of an admission rule
   #
-  $URI = qr{^/admission_rules/(\d+)(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('admission_rules/(\d+)');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $rule_id = $1;
@@ -1699,7 +1741,7 @@ SWITCH: for ($q) {
   #
   #{{{ POST /admission_rules : Create a new admission rule
   # 
-  $URI = qr{^/admission_rules(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('admission_rules');
   (OAR::API::POST( $_, $URI ) || OAR::API::PUT( $_, $URI )) && do {
     $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
@@ -1757,7 +1799,7 @@ SWITCH: for ($q) {
   #
   #{{{ DELETE /admission_rules/<id> : Delete an admission rule
   #
-  $URI = qr{^/admission_rules/(\d+)(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('admission_rules/(\d+)');
   OAR::API::DELETE( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $rule_id = $1;
@@ -1796,7 +1838,7 @@ SWITCH: for ($q) {
   #
   #{{{ POST /admission_rules/<id>[?method=delete]: Erase or Delete an admission rule
   #
-  $URI = qr{^/admission_rules/(\d+)(\.yaml|\.json|\.html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('admission_rules/(\d+)');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $rule_id = $1;
@@ -1874,7 +1916,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /config : List of all the configured variables
   #
-  $URI = qr{^/config\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('config');
   OAR::API::GET( $_, $URI ) && do {
   	$_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
@@ -1908,7 +1950,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /config/file : Get the raw configuration file
   #
-  $URI = qr{^/config/file\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('config/file');
   OAR::API::GET( $_, $URI ) && do {
   	$_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
@@ -1930,7 +1972,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /config/<variable_name> : Get a configuration variable value
   #
-  $URI = qr{^/config/(\w+)\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('config/(\w+?)');
   OAR::API::GET( $_, $URI ) && do {
   	$_->path_info =~ m/$URI/;
   	my $variable = $1;
@@ -1971,7 +2013,7 @@ SWITCH: for ($q) {
   #
   #{{{ POST /config/<variable_name> : Change the value of a configuration parameter
   #
-  $URI = qr{^/config/(\w+)\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('config/(\w+?)');
   OAR::API::POST( $_, $URI ) && do {
   	$_->path_info =~ m/$URI/;
   	my $variable = $1;
@@ -2026,7 +2068,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /desktop/agents : Desktop computing agent sign in
   #
-  $URI = qr{^/desktop/agents(.*)$};
+  $URI = OAR::API::uri_regex_html_json_yaml('desktop/agents');
   OAR::API::GET( $_, $URI ) && do {
 
     my $db = OAR::IO::connect() or die "cannot connect to the data base\n";
@@ -2060,7 +2102,7 @@ SWITCH: for ($q) {
   # 
   #{{{ GET /media/ls/<path> : List files
   #
-  $URI = qr{^/media/ls/(.*)$};
+  $URI = OAR::API::uri_regex_no_tail('media/ls/(.*)');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $path=$1;
@@ -2115,7 +2157,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /media/<file>?tail=<n> : Get a file (tail it to <n> lines if specified)
   #
-  $URI = qr{^/media/(.*)$};
+  $URI = OAR::API::uri_regex_no_tail('media/(.*)');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $filename=$1;
@@ -2168,7 +2210,7 @@ SWITCH: for ($q) {
   #
   #{{{ POST /media/chmod/<file>?mode=<mode> : Change the unix mode of a file
   #
-  $URI = qr{^/media/chmod/(.*)$};
+  $URI = OAR::API::uri_regex_no_tail('media/chmod/(.*)');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $filename = $1;
@@ -2221,10 +2263,11 @@ SWITCH: for ($q) {
   #
   #{{{ POST /media/<file> : Upload a file and create underlying directories
   #
-  $URI = qr{^/media/(.*)$};
+  $URI = OAR::API::uri_regex_no_tail('media(/force)?/(.*)');
   OAR::API::POST( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
-    my $filename=$1;
+    my $force=$1;
+    my $filename=$2;
     my $ext = OAR::API::set_ext($q,undef);
     (my $header, my $type) = OAR::API::set_output_format($ext);
 
@@ -2253,7 +2296,7 @@ SWITCH: for ($q) {
     $file =~ s|/~/|$user_infos[7]/|;  
 
     # Check if the file already exists
-    if (system("$OARDODO_CMD","test","-f","$file") == 0) {
+    if (system("$OARDODO_CMD","test","-f","$file") == 0 and ($force eq "" or not defined($force))) {
       OAR::API::ERROR(403, "File already exists", "The file already exists");
       last;
     }
@@ -2318,7 +2361,7 @@ SWITCH: for ($q) {
   #
   #{{{ DELETE /media/<file> : Delete a file or a directory recursively
   #
-  $URI = qr{^/media/(.*)$};
+  $URI = OAR::API::uri_regex_no_tail('media/(.*)');
   OAR::API::DELETE( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $filename=$1;
@@ -2365,7 +2408,7 @@ SWITCH: for ($q) {
   # 
   #{{{ GET /select_all?query=<query>) : Allows select SQL queries into the OAR database (ro) 
   #
-  $URI = qr{^/select_all\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('select_all(.*?)');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $ext=OAR::API::set_ext($q,$1);
@@ -2424,7 +2467,7 @@ SWITCH: for ($q) {
 
     print $header;
     print $HTML_HEADER if ($ext eq "html");
-    print OAR::API::export(\$result,$ext);
+    print OAR::API::export($result,$ext);
     OAR::IO::disconnect($dbh);
     last;
   };
@@ -2436,7 +2479,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /colmet/job/<id>?[from=timestamp]&[to=timestamp]&[metrics=m1,m2...] : Extract colmet data for the given job
   #
-  $URI = qr{^/colmet/job/(\d+)$};
+  $URI = OAR::API::uri_regex_no_tail('colmet/job/(\d+)');
   OAR::API::GET( $_, $URI ) && do {
     $_->path_info =~ m/$URI/;
     my $jobid = $1;
@@ -2523,7 +2566,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /index : Welcome page (html only)
   #
-  $URI = qr{^/index\.html$};
+  $URI = OAR::API::uri_regex_no_tail('index\.html');
   OAR::API::GET( $_, $URI ) && do {
     print $q->header( -status => 200, -type => "text/html" );
     print $HTML_HEADER;
@@ -2534,7 +2577,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /jobs/form : Html form for job posting
   #
-  $URI = qr{^/jobs/form.html$};
+  $URI = OAR::API::uri_regex_no_tail('form\.html');
   OAR::API::GET( $_, $URI ) && do {
     (my $header, my $type)=OAR::API::set_output_format("html");
     print $header;
@@ -2554,7 +2597,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /admission_rules/form : Html form for admission rules submission
   #
-  $URI = qr{^/admission_rules/form.html$};
+  $URI = OAR::API::uri_regex_no_tail('admission_rules/form\.html');
   OAR::API::GET( $_, $URI ) && do {
     (my $header, my $type)=OAR::API::set_output_format("html");
     print $header;
@@ -2572,7 +2615,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /resources/form : Html form for resources generation
   #
-  $URI = qr{^/resources/form.html$};
+  $URI = OAR::API::uri_regex_no_tail('resources/form\.html');
   OAR::API::GET( $_, $URI ) && do {
     (my $header, my $type)=OAR::API::set_output_format("html");
     print $header;
@@ -2595,7 +2638,7 @@ SWITCH: for ($q) {
   #
   #{{{ GET /stress_factor/<cluster_name> : return the stress factor of the given cluster
   #
-  $URI = qr{^/stress_factor\.*(yaml|json|html)*$};
+  $URI = OAR::API::uri_regex_html_json_yaml('stress_factor');
   OAR::API::GET( $_, $URI ) && do {
         $_->path_info =~ m/$URI/;
     my $ext = OAR::API::set_ext($q,$1);
