@@ -6,6 +6,7 @@ use Data::Dumper;
 use OAR::Version;
 use OAR::IO;
 use OAR::Conf qw(init_conf dump_conf get_conf is_conf);
+use OAR::Walltime qw(get);
 
 my $base;
 my $current_date = -1;
@@ -564,4 +565,9 @@ sub get_job_duration($$){
     return(sprintf("%i:%02i:%02i", $h,$m,$s));
 }
 
+sub get_job_walltime_change($){
+    my $jobid = shift;
+    return OAR::Walltime::get($base,$jobid);
+}
+
 1;
